"""Argparse Wrapper Class for automatic config loading"""
import argparse
import os

from .config import NETDBConfig, NETDBConfigError


class ArgumentParser(argparse.ArgumentParser):
    """Argument parser with default common arguments for NetDB-api cli
    tools. Includes default values."""

    def __init__(
            self,
            epilog="The Variables BASE_URL, VERSION and TOKEN can be loaded from config file,"
                   "environment or from command line arguments.\n"
                   "Variable precendence is as followed (from greatest to least,"
                   "means the first listed variable overwrites all other variables):\n"
                   "  1. cli-arguments\n"
                   "  2. environment variables\n"
                   "  3. config-file\n",
            formatter_class=argparse.RawTextHelpFormatter,
            **kwargs
    ):
        super().__init__(formatter_class=formatter_class, epilog=epilog, **kwargs)
        self.add_argument('--auth-config',
                          default=os.path.expanduser('~/.config/netdb_client.ini'),
                          help='config file path (default: %(default)s)')
        self.add_argument('--endpoint', '-e',
                          help='endpoint to use.\n'
                               'Environment: "NETDB_ENDPOINT"\n'
                               'Config: [DEFAULT]: endpoint')
        self.add_argument('--base-url', '-b',
                          help='webapi server.\n'
                               'Environment: "NETDB_BASE_URL"\n'
                               'Config: [$endpoint]: base_url')
        self.add_argument('--token', '-t',
                          help='user API token.\n'
                               'Environment: "NETDB_TOKEN"\n'
                               'Config: [$endpoint]: token')

    def parse_args(self, args=None, namespace=None):
        args = super().parse_args(args, namespace)
        try:
            config = NETDBConfig(args.auth_config)
        except NETDBConfigError as err:
            self.error(str(err))

        def load_config(option_name):
            """load config for an option"""

            # if value is already set, we can stop here
            if getattr(args, option_name) is not None:
                return

            envvar_name = f'NETDB_{option_name.upper()}'
            # try to load option from environment
            value = os.getenv(envvar_name, None)

            if value is None:
                # endpoint should be loaded from DEFAULT section
                if option_name == 'endpoint':
                    section = 'DEFAULT'
                # everything else loads from the $endpoint section
                else:
                    section = getattr(args, 'endpoint')
                # load value from config section
                value = config.get(section, option_name, fallback=None)

            # For backwards compatibility:
            # if base_url is still unknown use endpoint instead
            if option_name == 'base_url' and value is None:
                value = section

            if value is None:
                self.error(
                    f'No {option_name} specified (looked in args, environment variable '
                    f'"{envvar_name}", config-file in "{args.auth_config} section {section}")')

            # we have a value now and can use it
            setattr(args, option, value)

        # start with endpoint to get the right config section
        for option in ['endpoint', 'base_url', 'token']:
            load_config(option)

        return args
