"""click framework helper functions for config loading"""
import configparser
import importlib
import stat
from pathlib import Path

import click


def load_config_section_values(config: configparser.ConfigParser, endpoint: str) -> dict[str, str]:
    base_url = config.get(endpoint, 'base_url', fallback=None)
    if base_url is None:
        base_url = endpoint

    token = config.get(endpoint, 'token', fallback=None)

    return {'base_url': base_url, 'token': token}


def netdb_config_callback(ctx: click.Context, _param: click.Parameter, config_path: Path):
    if ctx.resilient_parsing:
        return

    config = configparser.ConfigParser()
    config_path_expanded = config_path.expanduser()
    if config_path_expanded.is_file():
        if config_path_expanded.stat().st_mode & stat.S_IRWXO:
            raise click.ClickException('Config file is readable by others. Please set it at least to 600 but never other-readable.', )
        config.read(config_path_expanded)

    obj = ctx.ensure_object(dict)
    obj['netdb-config'] = config  # TODO: keep internal state somewhere else

    endpoint = config.get('DEFAULT', 'endpoint', fallback=None)
    # if we have no endpoint in config, we can abort here
    if endpoint is None:
        return

    ctx.default_map = ctx.default_map or {}  # preserve existing defaults
    ctx.default_map['endpoint'] = endpoint

    config_options = load_config_section_values(config, endpoint)
    ctx.default_map.update(config_options)


def netdb_endpoint_callback(ctx: click.Context, _param: click.Parameter, endpoint: str):
    if ctx.resilient_parsing:
        return

    obj = ctx.find_object(dict)
    ctx.default_map = ctx.default_map or {}  # preserve existing defaults

    config_options = load_config_section_values(obj['netdb-config'], endpoint)
    ctx.default_map.update(config_options)

    return endpoint


def netdb_config_option(*param_decls, **kwargs):
    if not param_decls:
        param_decls = ('--netdb-config',)  # TODO this makes more sense IMHO

    kwargs.setdefault('callback', netdb_config_callback)
    kwargs.setdefault('is_eager', True)
    kwargs.setdefault('type', click.Path(path_type=Path))
    kwargs.setdefault('default', Path('~/.config/netdb_client.ini'))
    kwargs.setdefault('show_default', True)
    kwargs.setdefault('expose_value', False)
    kwargs.setdefault('help', 'netdb auth config file path')

    return click.option(*param_decls, **kwargs)


# TODO: should we really provide short options???
def netdb_endpoint_option(*param_decls, **kwargs):
    if not param_decls:
        param_decls = ('--endpoint', '-e', 'endpoint')

    kwargs.setdefault('callback', netdb_endpoint_callback)
    kwargs.setdefault('type', click.STRING)
    kwargs.setdefault('envvar', 'NETDB_ENDPOINT')
    kwargs.setdefault('show_envvar', True)
    kwargs.setdefault('show_default', True)
    kwargs.setdefault('expose_value', False)
    kwargs.setdefault('help', 'endpoint to use. Config: [DEFAULT]: endpoint')

    return click.option(*param_decls, **kwargs)


def netdb_base_url_option(*param_decls, **kwargs):
    if not param_decls:
        param_decls = ('--base-url', '-b', 'base_url')

    kwargs.setdefault('type', click.STRING)
    kwargs.setdefault('envvar', 'NETDB_BASE_URL')
    kwargs.setdefault('show_envvar', True)
    kwargs.setdefault('show_default', True)
    kwargs.setdefault('help', 'webapi server. Config: [$endpoint]: base_url')

    return click.option(*param_decls, **kwargs)


def netdb_token_option(*param_decls, **kwargs):
    if not param_decls:
        param_decls = ('--token', '-t', 'token')

    kwargs.setdefault('type', click.STRING)
    kwargs.setdefault('envvar', 'NETDB_TOKEN')
    kwargs.setdefault('show_envvar', True)
    kwargs.setdefault('show_default', False)
    kwargs.setdefault('help', 'user API token. Config: [$endpoint]: token')

    return click.option(*param_decls, **kwargs)


def netdb_api_options(target: str = 'netdb_api_endpoint', client_lib_version: str = None):
    if client_lib_version is None:
        client_lib_module_path = 'netdb_client'
    else:
        client_lib_module_path = f'netdb_client.api{client_lib_version.replace(".", "")}'

    api_lib = importlib.import_module(client_lib_module_path)

    def decorator(func):
        @netdb_config_option()
        @netdb_endpoint_option()
        @netdb_base_url_option()
        @netdb_token_option()
        def _decorator(*args, **kwargs):
            api_endpoint = api_lib.APIEndpoint(
                base_url=kwargs.pop('base_url'),
                token=kwargs.pop('token')
            )
            kwargs[target] = api_endpoint
            return func(*args, **kwargs)

        return _decorator

    return decorator
